@@ -1,11 +1,11 @@
 import pandas
-import neqsim
-from neqsim import java_gateway
-from neqsim import javaGateway
 from neqsim.standards import ISO6976
 import matplotlib.pyplot as plt
+import jpype
+import jpype.imports
+from jpype.types import *
+from neqsim.neqsimpython import neqsim
 
-neqsim = java_gateway.jvm.neqsim
 ThermodynamicOperations = neqsim.thermodynamicOperations.ThermodynamicOperations
 fluidcreator = neqsim.thermo.Fluid
 fluid_type = {
@@ -45,7 +45,31 @@ def fluid(name='srk', temperature=298.15, pressure=1.01325):
     fluid_function = fluid_type.get(name, neqsim.thermo.system.SystemSrkEos)
     return fluid_function(temperature, pressure)
 
-def fluid_df(reservoirFluiddf,lastIsPlusFraction=False):
+def readEclipseFluid(filename, wellName=''):
+    neqsim.thermo.util.readwrite.EclipseFluidReadWrite.pseudoName = wellName
+    fluid1 = neqsim.thermo.util.readwrite.EclipseFluidReadWrite.read(filename)
+    return fluid1
+
+def setEclipseComposition(fluid, filename, wellName=''):
+    neqsim.thermo.util.readwrite.EclipseFluidReadWrite.pseudoName = wellName
+    neqsim.thermo.util.readwrite.EclipseFluidReadWrite.setComposition(fluid, filename)
+
+def addFluids(fluids):
+    fluid = fluids[0].clone()
+    numberOfFluids = len(fluids)
+    for i in range(numberOfFluids-1):
+        fluid.addFluid(fluids[i+1])
+    return fluid
+
+def fluid_df(reservoirFluiddf,lastIsPlusFraction=False, autoSetModel=False, modelName=''):
+    if(autoSetModel):
+        fluidcreator.setAutoSelectModel(True)
+    else:
+        fluidcreator.setAutoSelectModel(False)
+    if(modelName):
+        fluidcreator.setThermoModel(modelName)
+    else:
+        fluidcreator.setAutoSelectModel(False)
     if 'MolarMass[kg/mol]' in reservoirFluiddf:
         definedComponentsFrame = reservoirFluiddf[reservoirFluiddf['MolarMass[kg/mol]'].isnull()]
     else:
@@ -59,61 +83,27 @@ def fluid_df(reservoirFluiddf,lastIsPlusFraction=False):
 def createfluid(fluid_type='dry gas'):
     return fluidcreator.create(fluid_type)
 
-def createfluid2(names, molefractions, unit="mol/sec"):
-    gateway = java_gateway#javaGateway.JavaGateway()
-    double_class = gateway.jvm.double
-    string_class = gateway.jvm.String
-    numberOfComponents =len(names)    
-    compositionJavaArray = gateway.new_array(double_class,numberOfComponents)
-    nameJavaArray = gateway.new_array(string_class,numberOfComponents)
-    i = 0
-    for i in range(0,numberOfComponents):
-        compositionJavaArray[i] = molefractions[i]
-        nameJavaArray[i] = names[i]
-        i = i+1
-    return fluidcreator.create2(nameJavaArray, compositionJavaArray, unit)
+def createfluid2(names, molefractions=None, unit="mol/sec"):
+    if(molefractions is None):
+        fluidcreator.create2(JString[:](names))
+    return fluidcreator.create2(JString[:](names), JDouble[:](molefractions), unit)
 
 def addOilFractions(fluid, charNames,molefractions,molarMass,  density, lastIsPlusFraction=False):
-    gateway = java_gateway
-    double_class = gateway.jvm.double
-    string_class = gateway.jvm.String
-    numberOfComponents =len(charNames)    
-    compositionJavaArray = gateway.new_array(double_class,numberOfComponents)
-    molarMassJavaArray = gateway.new_array(double_class,numberOfComponents)
-    relDensityJavaArray = gateway.new_array(double_class,numberOfComponents)
-    nameJavaArray = gateway.new_array(string_class,numberOfComponents)
-    i = 0
-    for i in range(0,numberOfComponents):
-        compositionJavaArray[i] = molefractions[i]
-        nameJavaArray[i] = charNames[i]
-        molarMassJavaArray[i] = molarMass[i]
-        relDensityJavaArray[i] = density[i]
-        i = i+1
     clonedfluid = fluid.clone()
-    clonedfluid = fluidcreator.addOilFractions(nameJavaArray, compositionJavaArray, molarMassJavaArray, relDensityJavaArray,lastIsPlusFraction)
+    clonedfluid = fluidcreator.addOilFractions(JString[:](charNames), JDouble[:](molefractions), JDouble[:](molarMass), JDouble[:](density),lastIsPlusFraction)
     return clonedfluid
 
 def newdatabase(system):
     system.createDatabase(1)
 
 def tunewaxmodel(fluid, experimentaldata):
-    gateway = java_gateway
-    double_class = gateway.jvm.double
-    tempList = experimentaldata['temperature']
+    tempList = [x+273.15 for x in experimentaldata['temperature']]
     presList = experimentaldata['pressure']
-    expList = experimentaldata['experiment']
-    numberofdata =len(tempList)
-    temperatureJavaArray = gateway.new_array(double_class,numberofdata)
-    pressureJavaArray = gateway.new_array(double_class,numberofdata)
-    experimentJavaArray = gateway.new_array(double_class,numberofdata, 1)
-    i = 0
-    for i in range(0,numberofdata):
-         temperatureJavaArray[i] = tempList[i]+273.15
-         pressureJavaArray[i] = presList[i]
-         experimentJavaArray[i][0] = expList[i]*100
+    expList = [x*100.0 for x in experimentaldata['experiment']]
+   
     waxsim = neqsim.PVTsimulation.simulation.WaxFractionSim(fluid)
-    waxsim.setTemperaturesAndPressures(temperatureJavaArray, pressureJavaArray)
-    waxsim.setExperimentalData(experimentJavaArray)
+    waxsim.setTemperaturesAndPressures(JDouble[:](tempList),JDouble[:](presList))
+    waxsim.setExperimentalData(JDouble[:](expList))
     waxsim.getOptimizer().setNumberOfTuningParameters(3)
     waxsim.getOptimizer().setMaxNumberOfIterations(20)
     waxsim.runTuning()
@@ -139,35 +129,16 @@ def dataFrame(system):
     return pandas.DataFrame(system.createTable(""))
 
 def calcproperties(gascondensateFluid, inputDict):
-    gateway = java_gateway
-    double_class = gateway.jvm.double
-    if("temperature" in inputDict and "pressure" in inputDict):
-         length =len(inputDict['temperature'])
-         pressureJavaArray = gateway.new_array(double_class,length)
-         temperatureJavaArray = gateway.new_array(double_class,length)
-         for i in range(0,length):
-             pressureJavaArray[i] = inputDict['pressure'][i]
-             temperatureJavaArray[i] = inputDict['temperature'][i]
-             i = i+1
-    properties = neqsim.util.generator.PropertyGenerator(gascondensateFluid, temperatureJavaArray, pressureJavaArray)
+    properties = neqsim.util.generator.PropertyGenerator(gascondensateFluid, JDouble[:](inputDict['temperature']), JDouble[:](inputDict['pressure']))
     props = properties.calculate()
     calculatedProperties= ({k: list(v) for k, v in props.items()})
     df = pandas.DataFrame(calculatedProperties)
     return df
 
 def separatortest(fluid, pressure, temperature, GOR=[], Bo=[], display=False):
-    gateway = java_gateway
-    double_class = gateway.jvm.double
     length =len(pressure)
-    pressureJavaArray = gateway.new_array(double_class,length)
-    temperatureJavaArray = gateway.new_array(double_class,length)
-    i = 0
-    for i in range(0,length):
-        pressureJavaArray[i] = pressure[i]
-        temperatureJavaArray[i] = temperature[i]
-        i = i+1
     sepSim = neqsim.PVTsimulation.simulation.SeparatorTest(fluid)
-    sepSim.setSeparatorConditions(temperatureJavaArray, pressureJavaArray)
+    sepSim.setSeparatorConditions(JDouble[:](temperature), JDouble[:](pressure))
     sepSim.runCalc()
     for i in range(0,length):
         GOR.append(sepSim.getGOR()[i])
@@ -185,16 +156,9 @@ def separatortest(fluid, pressure, temperature, GOR=[], Bo=[], display=False):
         plt.figure()
 
 def CVD(fluid, pressure, temperature, relativeVolume=[],liquidrelativevolume=[], Zgas=[],Zmix=[],cummulativemolepercdepleted=[], display=False):
-    gateway = java_gateway
-    double_class = gateway.jvm.double
     length =len(pressure)
-    pressureJavaArray = gateway.new_array(double_class,length)
-    i = 0
-    for i in range(0,length):
-        pressureJavaArray[i] = pressure[i]
-        i = i+1
     cvdSim = neqsim.PVTsimulation.simulation.ConstantVolumeDepletion(fluid)
-    cvdSim.setPressures(pressureJavaArray)
+    cvdSim.setPressures(JDouble[:](pressure))
     cvdSim.setTemperature(temperature)
     cvdSim.runCalc()
     for i in range(0,length):
@@ -216,24 +180,14 @@ def CVD(fluid, pressure, temperature, relativeVolume=[],liquidrelativevolume=[],
         plt.figure()
 
 def viscositysim(fluid, pressure, temperature, gasviscosity=[], oilviscosity=[],aqueousviscosity=[], display=False):
-    gateway = java_gateway
-    double_class = gateway.jvm.double
     length =len(pressure)
-    pressureJavaArray = gateway.new_array(double_class,length)
-    temperatureJavaArray = gateway.new_array(double_class,length)
-    i = 0
-    for i in range(0,length):
-        pressureJavaArray[i] = pressure[i]
-        temperatureJavaArray[i] = temperature[i]
-        i = i+1
     cmeSim = neqsim.PVTsimulation.simulation.ViscositySim(fluid)
-    cmeSim.setTemperaturesAndPressures(temperatureJavaArray, pressureJavaArray)
+    cmeSim.setTemperaturesAndPressures(JDouble[:](temperature), JDouble[:](pressure))
     cmeSim.runCalc()
     for i in range(0,length):
         gasviscosity.append(cmeSim.getGasViscosity()[i])
         oilviscosity.append(cmeSim.getOilViscosity()[i])
         aqueousviscosity.append(cmeSim.getAqueousViscosity()[i])
-        i = i+1
     if display:
         plt.figure()
         plt.plot(pressure, gasviscosity, "o")
@@ -246,18 +200,9 @@ def viscositysim(fluid, pressure, temperature, gasviscosity=[], oilviscosity=[],
         plt.figure()
 
 def CME(fluid, pressure, temperature, saturationPressure, relativeVolume=[], liquidrelativevolume=[], Zgas=[], Yfactor=[], isothermalcompressibility=[], density=[],Bg=[], viscosity=[], display=False):
-    gateway = java_gateway
-    double_class = gateway.jvm.double
     length =len(pressure)
-    pressureJavaArray = gateway.new_array(double_class,length)
-    temperatureJavaArray = gateway.new_array(double_class,length)
-    i = 0
-    for i in range(0,length):
-        pressureJavaArray[i] = pressure[i]
-        temperatureJavaArray[i] = temperature[i]
-        i = i+1
     cvdSim = neqsim.PVTsimulation.simulation.ConstantMassExpansion(fluid)    
-    cvdSim.setTemperaturesAndPressures(temperatureJavaArray, pressureJavaArray)
+    cvdSim.setTemperaturesAndPressures(JDouble[:](temperature), JDouble[:](pressure))
     cvdSim.runCalc()
     saturationPressure=cvdSim.getSaturationPressure()
     for i in range(0,length):
@@ -282,16 +227,9 @@ def CME(fluid, pressure, temperature, saturationPressure, relativeVolume=[], liq
         plt.figure()
 
 def difflib(fluid, pressure, temperature, relativeVolume = [], Bo=[], Bg=[], relativegravity=[], Zgas=[], gasstandardvolume=[], Rs=[], oildensity=[], gasgravity=[], display=False):
-    gateway = java_gateway
-    double_class = gateway.jvm.double
     length =len(pressure)
-    pressureJavaArray = gateway.new_array(double_class,length)
-    i = 0
-    for i in range(0,length):
-        pressureJavaArray[i] = pressure[i]
-        i = i+1
     cvdSim = neqsim.PVTsimulation.simulation.DifferentialLiberation(fluid)
-    cvdSim.setPressures(pressureJavaArray)
+    cvdSim.setPressures(JDouble[:](pressure))
     cvdSim.setTemperature(temperature)
     cvdSim.runCalc()
     for i in range(0,length):
@@ -318,18 +256,9 @@ def difflib(fluid, pressure, temperature, relativeVolume = [], Bo=[], Bg=[], rel
         plt.figure()
 
 def GOR(fluid, pressure, temperature, GORdata=[], Bo=[],  display=False):
-    gateway = java_gateway
-    double_class = gateway.jvm.double
     length =len(pressure)
-    pressureJavaArray = gateway.new_array(double_class,length)
-    temperatureJavaArray = gateway.new_array(double_class,length)
-    i = 0
-    for i in range(0,length):
-        pressureJavaArray[i] = pressure[i]
-        temperatureJavaArray[i] = temperature[i]
-        i = i+1
     GOR = neqsim.PVTsimulation.simulation.GOR(fluid)
-    GOR.setTemperaturesAndPressures(temperatureJavaArray, pressureJavaArray)
+    GOR.setTemperaturesAndPressures(JDouble[:](temperature), JDouble[:](pressure))
     GOR.runCalc()
     for i in range(0,length):
         GORdata.append(GOR.getGOR()[i])
@@ -349,19 +278,11 @@ def saturationpressure(fluid, temperature=-1.0):
     return cvdSim.getSaturationPressure()
 
 def swellingtest(fluid, fluid2, temperature, cummulativeMolePercentGasInjected, pressure = [], relativeoilvolume=[], display=False):
-    gateway = java_gateway
-    double_class = gateway.jvm.double
     length2 =len(cummulativeMolePercentGasInjected)
-    cummulativeMolePercentGasInjectedJava = gateway.new_array(double_class,length2)
-    i = 0
-    for i in range(0,length2):
-        cummulativeMolePercentGasInjectedJava[i] = cummulativeMolePercentGasInjected[i]
-        i = i+1
-    
     cvdSim = neqsim.PVTsimulation.simulation.SwellingTest(fluid)
     cvdSim.setInjectionGas(fluid2)
     cvdSim.setTemperature(temperature)
-    cvdSim.setCummulativeMolePercentGasInjected(cummulativeMolePercentGasInjectedJava)
+    cvdSim.setCummulativeMolePercentGasInjected(JDouble[:](cummulativeMolePercentGasInjected))
     cvdSim.runCalc()
     for i in range(0,length2):
         relativeoilvolume.append(cvdSim.getRelativeOilVolume()[i])
@@ -472,6 +393,10 @@ def TPflash(testSystem):
     testFlash = ThermodynamicOperations(testSystem)
     testFlash.TPflash()
     testSystem.init(3)
+
+def TPgradientFlash(testSystem, height, temperature):
+    testFlash = ThermodynamicOperations(testSystem)
+    testFlash.TPgradientFlash(height,temperature)
     
 def TVflash(testSystem, volume, unit="m3"):
     testFlash = ThermodynamicOperations(testSystem)
@@ -498,6 +423,12 @@ def VUflash(testSystem, volume, energy, unitVol= "m3", unit="J"):
     testFlash.VUflash(volume, energy, unitVol, unit)
     testSystem.init(3)
     
+def PUflash(testSystem, pressure, energy, unitPressure= "bara", unitEnergy="J"):
+    testFlash = ThermodynamicOperations(testSystem)
+    testSystem.setPressure(pressure, unitPressure) 
+    testFlash.PUflash(energy, unitEnergy)
+    testSystem.init(3)
+    
 def PVTpropTable(fluid1, fileName, lowTemperature, highTemperature, Tsteps, lowPressure, highPressure, Psteps):
     testFlash = ThermodynamicOperations(fluid1)
     testFlash.OLGApropTable(lowTemperature, highTemperature, Tsteps, lowPressure, highPressure, Psteps, fileName, 0)
@@ -630,27 +561,11 @@ def phaseenvelope(testSystem, plot=False):
     return testFlash
 
 def fluidComposition(testSystem, composition):
-    gateway = java_gateway
-    double_class = gateway.jvm.double
-    numberOfComponents =len(composition)    
-    compositionJavaArray = gateway.new_array(double_class,numberOfComponents)
-    i = 0
-    for i in range(0,numberOfComponents):
-        compositionJavaArray[i] = composition[i]
-        i = i+1
-    testSystem.setMolarComposition(compositionJavaArray)
+    testSystem.setMolarComposition(JDouble[:](composition))
     testSystem.init(0)
 
 def fluidCompositionPlus(testSystem, composition):
-    gateway = java_gateway
-    double_class = gateway.jvm.double
-    numberOfComponents =len(composition)    
-    compositionJavaArray = gateway.new_array(double_class,numberOfComponents)
-    i = 0
-    for i in range(0,numberOfComponents):
-        compositionJavaArray[i] = composition[i]
-        i = i+1
-    testSystem.setMolarCompositionPlus(compositionJavaArray)
+    testSystem.setMolarCompositionPlus(JDouble[:](composition))
     testSystem.init(0)
 
 def getExtThermProp(function, thermoSystem, t=0, p=0):
